import { isAsyncIterable } from "./is-async-iterable.js";
export function dropUntilAsync(iterable, predicate) {
    if (isAsyncIterable(iterable)) {
        return dropUntilAsyncIterable(iterable, predicate);
    }
    else {
        return dropUntilIterable(iterable, predicate);
    }
}
async function* dropUntilAsyncIterable(iterable, predicate) {
    var _a;
    const iterator = iterable[Symbol.asyncIterator]();
    let done;
    try {
        let index = 0;
        let value;
        while ({ value, done } = await iterator.next(), !done) {
            if (await predicate(await value, index++))
                break;
        }
        while (!done) {
            yield value;
            ({ value, done } = await iterator.next());
        }
    }
    finally {
        if (!done)
            await ((_a = iterator.return) === null || _a === void 0 ? void 0 : _a.call(iterator));
    }
}
async function* dropUntilIterable(iterable, predicate) {
    var _a;
    const iterator = iterable[Symbol.iterator]();
    let done;
    try {
        let index = 0;
        let value;
        while ({ value, done } = iterator.next(), !done) {
            if (await predicate(await value, index++))
                break;
        }
        while (!done) {
            yield value;
            ({ value, done } = iterator.next());
        }
    }
    finally {
        if (!done)
            (_a = iterator.return) === null || _a === void 0 ? void 0 : _a.call(iterator);
    }
}
//# sourceMappingURL=drop-until-async.js.map