import { go } from '@blackglory/go';
import { assert } from '@blackglory/errors';
import { Queue } from '@blackglory/structures';
import { ReusableDeferred } from 'extra-promise';
import { AbortController } from 'extra-abort';
export function prefetchAsync(iterable, size) {
    assert(Number.isInteger(size), 'The parameter size must be an integer');
    assert(size > 0, 'The parameter size must be greater than 0');
    let value;
    let done;
    const iterator = iterable[Symbol.asyncIterator]();
    const buffer = new Queue();
    const controller = new AbortController();
    const consumed = new ReusableDeferred();
    const prefetched = new ReusableDeferred();
    queueMicrotask(prefetch);
    return go(async function* () {
        var _a;
        try {
            while (true) {
                while (buffer.size) {
                    const value = buffer.dequeue();
                    consumed.resolve();
                    yield value;
                }
                if (done)
                    break;
                await prefetched;
            }
        }
        finally {
            controller.abort();
            consumed.reject(controller.signal.reason);
            if (!done)
                await ((_a = iterator.return) === null || _a === void 0 ? void 0 : _a.call(iterator));
        }
    });
    async function prefetch() {
        try {
            while (true) {
                if (controller.signal.aborted) {
                    prefetched.reject(controller.signal.reason);
                    break;
                }
                if (buffer.size < size) {
                    ({ value, done } = await iterator.next());
                    if (done) {
                        prefetched.resolve();
                        break;
                    }
                    else {
                        buffer.enqueue(value);
                        prefetched.resolve();
                    }
                }
                else {
                    await consumed;
                }
            }
        }
        catch (err) {
            prefetched.reject(err);
        }
    }
}
//# sourceMappingURL=prefetch-async.js.map