import { isUndefined } from 'extra-utils';
export function reduce(iterable, fn, initialValue) {
    if (isUndefined(initialValue)) {
        return reduceWithoutInitialValue(iterable, fn);
    }
    else {
        return reduceWithInitialValue(iterable, fn, initialValue);
    }
}
function reduceWithInitialValue(iterable, fn, initialValue) {
    let result = initialValue, index = 0;
    for (const currentValue of iterable) {
        result = fn(result, currentValue, index++);
    }
    return result;
}
function reduceWithoutInitialValue(iterable, fn) {
    var _a;
    const iterator = iterable[Symbol.iterator]();
    let done;
    try {
        let result = readInitialValue(iterator);
        let index = 1;
        let value;
        while ({ value, done } = iterator.next(), !done) {
            result = fn(result, value, index++);
        }
        return result;
    }
    finally {
        if (!done)
            (_a = iterator.return) === null || _a === void 0 ? void 0 : _a.call(iterator);
    }
    function readInitialValue(iterator) {
        const result = iterator.next();
        if (result.done) {
            done = true;
            throw new Error('Reduce of empty iterable with no initial value');
        }
        return result.value;
    }
}
//# sourceMappingURL=reduce.js.map